"""
This file provides a more uniform interface to gym.make(env_id) that handles
imports and normalization
"""

import gym

from rlkit.envs.wrappers import NormalizedBoxEnv
from rlkit.core.logging import bold


def domain_to_env(name):

    from gym.envs.mujoco import (
        HalfCheetahEnv,
        InvertedPendulumEnv,
        HumanoidEnv,
        HopperEnv,
        AntEnv,
        Walker2dEnv,
    )

    return {
        "invertedpendulum": InvertedPendulumEnv,
        "humanoid": HumanoidEnv,
        "halfcheetah": HalfCheetahEnv,
        "hopper": HopperEnv,
        "ant": AntEnv,
        "walker2d": Walker2dEnv,
    }[name]


def env_producer(
    env_id=None,
    env_class=None,
    env_kwargs=None,
    normalize_env=True,
    seed=None,
    d4rl=False,
):
    assert env_id or env_class
    if env_class and env_kwargs is None:
        env_kwargs = {}

    if d4rl:
        bold("Using D4RL!")
        assert normalize_env == False
        import d4rl

        env = gym.make(env_id)
        env = env.env  # Unwrap TimeLimit on Environment
    elif env_class:
        env = env_class(**env_kwargs)
    elif env_id:
        env = gym.make(env_id)
    if seed is not None:
        env.seed(seed)
    if normalize_env:
        env = NormalizedBoxEnv(env)

    return env
